/** Klasse, die einen reinen Autozug beschreibt. Unterklasse von Zug
 * Es ist nicht mglich, Personenwaggons einzuhngen (denkt man).
 *
 * @author Kirsten
 * @version 1.0
 */

//Diese Klasse gehrt zum package zuege und bentigt das package waggons.

package zuege;
import zuege.waggons.*;

//Diese Klasse ist final und kann deshalb nicht mehr als Oberklasse fr
//Vererbung verwendet werden.

final public class Autozug extends Zug
{
    /** Konstruktor fr den Autozug
     * @param name Name des Zugs
     * @param l Lokomotive fr den Zug
     */
    public Autozug(String name, Lokomotive l)
    {
	//bergabe der Parameter an den Konstruktor der Oberklasse
	super(name, l);
    }

    /** Konstruktor zum Umhngen von Zgen
     * @param name Name des Zugs
     * @param l Lokomotive fr den Zug
     * @param z ursprnglicher Zug, dessen Waggons bernommen werden sollen
     */
    public Autozug(String name, Lokomotive l, Autozug z)
    {
	//bergabe an den Konstruktor der Oberklasse
	super(name, l, z);
    }

    /** Methode zum Anhngen von Waggons an einen Autozug,
     * berldt die Methode der Oberklasse dahingehend, dass lediglich
     * Autowaggons angehngt werden knnen
     * @param w Waggon, der an den Zug angehngt werden soll
     */
    public void anhaengen(Autowaggon w)
    {
	//ruft lediglich Methode der Oberklasse auf, das die Funktionalitt
	//nicht verndert werden muss
	super.anhaengen(w);

	//wir machen hier noch eine Ausgabe, um die unterschiedlichen 
	//Methoden von Ober-und Unterklasse unterscheiden zu knnen
	System.out.println("Autozug: Waggon angehngt");
    }

    /** Methode zum Anhngen von Waggons an einen Autozug,
     * berldt die Methode der Oberklasse dahingehend, dass lediglich
     * Autowaggons angehngt werden knnen
     * @param w Waggon, der an den Zug angehngt werden soll
     */
    public void anhaengen(Personenwaggon w)
    {
	//wir machen hier nur eine Ausgabe, um die unterschiedlichen 
	//Methoden von Ober-und Unterklasse unterscheiden zu knnen
	System.out.println("Autozug: Waggon nicht angehngt");
    }

    /** Methode, die die Anzahl von Sitzpltzen im Zug liefert,
     * berschreibt die Methode der Oberklasse dahingehend, dass immer 0
     * zurckgeliefert wird, da es im Autozug keine Sitzpltze gibt
     * @return 0
     */
    public int getPltze()
    {
	return 0;
    }

    /** Methode, die den Autozug als String darstellt
     * @return Daten des Autozugs: Lnge und Anzahl der Stellpltze
     */
    public String toString()
    {
	String ergebnis = "";
	//Variable, um sich von Waggon zu Waggon durch den Zug zu hangeln
	Waggon waggon;

	//Zug mit eventuellem Namen ausgeben
	ergebnis += "Autozug ";

	//Achtung: Zug hat einen private-Zugriff auf das Attribut Name
	//deshalb kann das Attribut nicht direkt verwendet werden, sondern
	//muss ber getName() abgefragt werden
	if(getName() != null)
	    ergebnis += getName() + " ";

	//Gesamtdaten des Zugs: Lnge, Anzahl der Stellpltze
	//fr die Lnge gilt ebenfalls: private-Variable von Zug, deshalb
	//nur ber die Methode getLaenge() erreichbar
	ergebnis += "der Laenge " + getLaenge() + " mit " + getAutos() + 
	    " Plaetzen fuer " +
	    "Autos\nBestehend aus:\n";

	//ist die Lok da?
	if(getLokomotive() != null)
	{
	    //Daten der Lok ausgeben
	    ergebnis += getLokomotive();
	    ergebnis += "\nAnzahl der Waggons: " + getAnzahl();
	    //sind Waggons da
	    waggon = getLokomotive().getAnfang();

	    //Daten aller Waggons anzeigen
	    if(waggon != null)
	    {
		ergebnis += "\n";
		ergebnis += waggon;
		
		while(waggon.getNaechster() != null)
		{
		    ergebnis += "\n";
		    ergebnis += waggon.getNaechster();

		    waggon = waggon.getNaechster();
		}
	    }
	}
	return ergebnis;
    }

    /** Main-Methode zum Testen*/
    public static void main(String args[])
    {
	Lokomotive steel = new Lokomotive("Steel Horse", 25f, 5312);
	Autozug rioGrande = new Autozug("Rio Grande Express", steel);

	//zunchst einmal legen wir einen Autozug an und rufen dessen
	//Methoden auf
	//alles luft wie erwartet
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	rioGrande.anhaengen(new Autowaggon(25, 15f));
	rioGrande.anhaengen(new Autowaggon(125, 23f));

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande);

	rioGrande.abhaengen();
	rioGrande.abhaengen();
	rioGrande.abhaengen();

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//nun legen wir einen ganz normalen Zug an und rufen dessen
	//Methoden aus
	//alles luft wie erwartet
	Lokomotive mathilda = new Lokomotive("Mathilda", 12.5f, 1111); 
	Zug rioGrande2 = new Zug("Rio Grande Express 2", mathilda);

	System.out.println(rioGrande2);

	rioGrande2.anhaengen(new Personenwaggon(25, 15f));
	rioGrande2.anhaengen(new Personenwaggon(125, 23f));
	rioGrande2.anhaengen(new Autowaggon(125, 23f));

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande2);

	rioGrande2.abhaengen();
	rioGrande2.abhaengen();
	rioGrande2.abhaengen();

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande2);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//Zug ist die Oberklasse von Autozug
	//die die Unterklassen immer konform zur Oberklasse sein mssen,
	//ist es vllig korrekt ein Objekt vom Typ Zug anzulegen und
	//ihm einen Autozug zuzuweisen
	Lokomotive mary = new Lokomotive("Fat Mary", 28.12f, 1112);
	Zug rioGrande3 = new Autozug("Rio Grande Express 3", mary);

	//in dieser Ausgabe sehen wir, dass es sich um einen Autozug
	//handelt (berschriebene Methode toString() wird verwendet)
	System.out.println(rioGrande3);
	
	//nun hngen wir Waggons an
	//wir sehen nun, dass die zustzliche Ausgabe der Methode 
	//anhaengen() nicht erfolgt
	//scheinbar haben wir es nun mit einem Zug zu tun!
	rioGrande3.anhaengen(new Autowaggon(25, 15f));
	rioGrande3.anhaengen(new Autowaggon(125, 23f));
	rioGrande3.anhaengen(new Personenwaggon(125, 23f));

	//es handelt sich um einen typischen Fall von dynamischen Typen
	//bzw. dynamischen Binden bei Vererbung
	//es wird erst zur Laufzeit bestimmt, welche Methode verwendet wird,
	//in der nun folgenden Ausgabe hingegen, wird wieder die Methode
	//toString() von Autozug verwendet, da sie passt
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande3);

	rioGrande3.abhaengen();

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	
	System.out.println(rioGrande3);

	
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//nun wollen wir noch einen neuen Zug erzeugen, indem wir die
	//Waggons umhngen
	//hier muss der ursprngliche Zug ein Autozug sein, da der
	//Konstruktor mit drei Parametern dahingehend gendert wurde
		
	//zunchst einmal hngen wir wieder Waggons an den rioGrande
	//da dieser Zug als Autozug deklariert wurde, wird diesmal
	//beim Versuch einen Personenwaggon anzuhngen, die Methode anhaengen()
	//von Autozug verwendet und der Personenwaggon nicht angehngt
	
	rioGrande.anhaengen(new Personenwaggon(113, 35.87f));
	rioGrande.anhaengen(new Autowaggon(19, 5f));

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande);
	
	Lokomotive steam = new Lokomotive("Steam Waltz", 25.4f, 5333);
	Zug orient = new Autozug("OrientExpress", steam, rioGrande);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(rioGrande);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(orient);	

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	
    }
}
