/** Klasse, die den Zug beschreibt. Ein Zug beginnt mit einer Lokomotive, 
 * daran hngen die Waggons. 
 *
 * @author Kirsten
 * @version 1.0
 */

//Diese Klasse gehrt zum package zuege und bentigt das package waggons
package zuege;
import zuege.waggons.*;

//Diese Klasse ist nicht final, sie dient als Superklasse von Personenzug.

public class Zug
{
    //jeder Zug besteht zunchst aus einer Lokomotive
    private Lokomotive lok;
    //jeder Zug hat einen Namen
    private String name;

    //Zhler fr die Waggons eines Zugs
    private int waggons;

    /** Konstrukor fr einen Zug
     * @param name Name des Zugs
     * @param l Lokomotive fr den Zug
     */
    public Zug(String name, Lokomotive l)
    {
	lok = l;
	this.name = new String(name);

	waggons = 0;
    }

    /** Konstruktor fr einen Zug, der einen weiteren Zug bergeben bekommt
     * die Idee ist, dass die neue Lokomotive von hinten an den Zug fhrt
     * die Waggons dort angekoppelt werden und von dem anderen Zug abgekoppelt
     * der neue Zug enthlt also die gleichen wie der Zug vorher, nur dass
     * die Waggons eine andere Reihenfolge haben, die Lokomotive des 
     * anderen Zugs hat keine Waggons mehr
     * @param name Name des neuen Zugs
     * @param r Lokomotive des neuen Zugs
     * @param x der alte Zug
     */
    public Zug(String name, Lokomotive r, Zug x)
    {
	this(name, r);
	    
        //waggons entsprechen denen von Zug x
	waggons = x.getAnzahl();
	x.resetAnzahl();

	//nu die Waggons in umgekehrter Reihenfolge anhngen
	Waggon w = umhaengen(x.getLokomotive().getAnfang());
	r.setAnfang(w);
	//an der alten Lokomotive abhngen
	x.getLokomotive().setAnfang(null);
    }

    /** Anhngen eines Waggons an das Ende eines Zugs
     * @param w Waggon, der angehngt werden soll
     */
    public void anhaengen(Waggon w)
    {
	//Variable zum Speichern eines Waggons
	//wird bentigt, um den letzten Waggon zu finden, an den
	//der neue dann angehngt wrid
	Waggon letzter = null;

	//ist die Lok da?
	if(lok != null)
	{
	    //ist ein Waggon anzuhngen?
	    if(w != null)
	    {
		//nehme den ersten Waggon des Zugs
		letzter = lok.getAnfang();
	
		//wenn hier bereits einer hngt, suche weiter nach dem
		//letzten Waggon
		if(letzter != null)
		{
		    //suche solange, bis der letzte Waggon gefunden wird
		    while(letzter.getNaechster() != null)
			letzter = letzter.getNaechster();

		    //neuen Waggon anhngen
		    letzter.setNaechster(w);
		}
		//dies ist der erste Waggon, direkt an die Lokomotive hngen
		else
		    lok.setAnfang(w);
		
		//Zhler erhhen, nu ist ein Waggon mehr am Zug
		waggons++;
	    }  
	}
    }

    /** Methode, um den letzten Waggon abzuhngen
     */
    public void abhaengen()
    {
	//Variable, um sich den vorletzten Waggon zu merken
	//(der letzte soll ja abgehngt werden)
	Waggon vorletzter = null;

	//ist die Lok da?
	if(lok != null)
	{
	    //zunchst einmal den ersten Wagen an der Lok anschauen
	    vorletzter = lok.getAnfang();
	    
	    //es gibt mindestens einen Waggon, der abgehngt werden kann
	    if(vorletzter != null)
	    {
		//der erste Waggon wird abgehngt, da es nur genau diesen
		//einen gibt
		if(vorletzter.getNaechster() == null)
		{
		    lok.setAnfang(null);
		}
		//wir mssen den vorletzten Waggon suchen
		//und dann den letzten abhngen, sprich null setzen
		else
		{
		    while(vorletzter.getNaechster().getNaechster() != null)
			vorletzter = vorletzter.getNaechster();

		    vorletzter.setNaechster(null);
		}
		//es wurde ein Waggon abgehngt, drum wird der Zhler 
		//erniedrigt
		waggons --;
	    }
	}
    }

    /** Methode, die alle Waggons in der umgekehrten Reihenfolge zurckliefert
     * dies wird bentigt, wenn am Ende des Zugs eine neue Lokomotive angehngt
     * und die ursprngliche Lokomotive abgehngt wird, danach hngen die
     * Waggons in umgekehrter Reihenfolge an er neuen Lok
     * umhngen kann nur intern vorgenommen werden, deshalb ist die Methdode
     * private
     * @param l Waggon, der vormals der erste und am Schluss der Methode der 
     * letzte ist
     * @return der neue erste Waggon
     */
    private Waggon umhaengen(Waggon l)
    {
	//der letzte Waggon des ursprnglichen Zugs wird erster des neuen
	//um keinen Waggon zu verlieren, brauchen wir Hilfsvariablen
	Waggon w = null;
	Waggon h1 = null;
	Waggon h2 = null;

        //wurde ein Waggon bergeben?
	if(l != null)
	{
	    //ersten Waggon merken
	    h1 = l;
		
	    //nchster Waggon
	    w = h1.getNaechster();

	    //der allererste Wagen ist hinterher der letzte und hat
	    //keinen Vorgnger mehr
	    h1.setNaechster(null);

	    //kein Waggon?
	    //dann mssen wir nur den ersten zurckgeben
	    if(w == null)
		return h1;
		
	    //berprfen, ob da noch ein Waggon ist
	    while(w.getNaechster() != null)
	    {	     
		//sind wir schon am Ende angekommen? nein ->
		//folgenden Waggon merken, um ihn nicht zu verlieren
		h2 = w.getNaechster();
		//nun die Waggons umhngen
		w.setNaechster(h1);
		//Anfang der Waggons neu setzen
		h1 = w;
		w = h2;
	    }

	    //letzter Waggon (beim neuen Zug der erste) ist gefunden
	    //mit den restlichen Waggons verbinden
	    w.setNaechster(h1);
	}
	return w;
    }

    /** Methode, die gesamte Anzahl von Sitzpltzen im Zug berechnet
     * @return Anzahl von Sitzpltzen im Zug
     */
    public int getPersonen()
    {
	//Zhler fr die Sitzpltze
	int anzahl = 0;
	//Variable, um sich von Waggon zu Waggon durch den Zug zu hangeln
	Waggon letzter;

	//ist die Lok da?
	if(lok != null)
	{
	    //ersten Waggon nehmen
	    letzter = lok.getAnfang();
	    //ist ein Waggon da?
	    if(letzter != null)
	    {
		//berprfe jeden Waggon bis zum Ende
		while(letzter.getNaechster() != null)
		{
		    //handelt es sich um einen Personenwagen?
		    //dann mssen wir die Anzahl hochzhlen
		    if(letzter instanceof Personenwaggon)
		    {
			anzahl += ((Personenwaggon)letzter).getPlaetze();
		    }
		    letzter = letzter.getNaechster();
		}
		//nun noch die Personen des letzten Waggons addieren
		if(letzter instanceof Personenwaggon)
		    anzahl += ((Personenwaggon)letzter).getPlaetze();
	    }
	}
	return anzahl;
    }
    
    /** Methode, die die gesamte Anzahl von Stellpltzen fr Autos im Zug
     * berechnet
     * @return Anzahl der Stellpltze im Zug
     */
    public int getAutos()
    {
	//Anzahl von Stellpltzen
	int anzahl = 0;
	//Variable, um sich von Waggon zu Waggon durch den Zug zu hangeln
	Waggon letzter;

	//ist die Lok da?
	if(lok != null)
	{
	    //ersten Waggon nehmen
	    letzter = lok.getAnfang();
	    //ist ein Waggon da?
	    if(letzter != null)
	    {
		//bis zum letzten Waggon durchhangeln
		while(letzter.getNaechster() != null)
		{
		    //handelt es sich um einen Autowaggon?
		    //dann mssen wir die Anzahl entsprechend hochzhlen
		    if(letzter instanceof Autowaggon)
			anzahl += ((Autowaggon)letzter).getAnzahl();
		    letzter = letzter.getNaechster();
		}
		//zuletzt die Stellpltze im letzten Waggon draufrechnen
		if(letzter instanceof Autowaggon)
		    anzahl += ((Autowaggon)letzter).getAnzahl();
	    }
	}
	return anzahl;
    }

    /** Methode, die die Gesamtlnge des Zugs aus den Einzellngen der
     * Lokomotive und der Waggons berechnet
     * @return Lnge des Zugs
     */
    public float getLaenge()
    {
	//Lnge des Zugs
	float laenge = 0;
	//Variable, um sich von Waggon zu Waggon durch den Zug zu hangeln
	Waggon letzter;

	//ist die Lok da?
	if(lok != null)
	{
	    //Lnge der Lok addieren
	    laenge += lok.getLaenge();
	    //ersten Waggon suchen
	    letzter = lok.getAnfang();
	    //ist ein Waggon da?
	    if(letzter != null)
	    {
		//die Lnge von jedem Waggon hinzuaddieren
		while(letzter.getNaechster() != null)
		{
		    laenge += letzter.getLaenge();
		    letzter = letzter.getNaechster();
		}
		laenge += letzter.getLaenge();
	    }
	}
	return laenge;
    }

    /** Methode, die die Anzahl der Waggons zurckgibt
     * @return Anzahl von Waggons im Zug
     */
    final public int getAnzahl()
    {
	return waggons;
    }

    /** Methode, um die Anzahl der Waggons zu setzen,
     *	falls der Zug komplett aufgelst wird
     *  diese Methode ist private, weil das nur intern geschehen kann
     */
    final private void resetAnzahl()
    {
	waggons = 0;
    }

    /** Methode, die die Lokomotive zurckliefert
     * @return Lokomotive
     */
    final public Lokomotive getLokomotive()
    {
	return lok;
    }

    /** Methode, die den Namen zurckliefert
     * @return Name des Zugs
     */
    final public String getName()
    {
	return name;
    }
    
    
    /** Methode zur Ausgabe eines Zugs als Zeichenkette 
     * @return Zeichenkette die smtliche Angaben zum Zug liefert
     */
    public String toString()
    {
	String ergebnis = "";
	Waggon waggon;

	//Zug mit eventuellem Namen ausgeben
	ergebnis += "Zug ";

	if(name != null)
	    ergebnis += name + " ";

	//Gesamtdaten des Zugs: Lnge, Anzahl der Stellpltze, Anzahl
	//der Sitzpltze
	ergebnis += "der Laenge " + getLaenge() + " mit " + getAutos() + 
	    " Plaetzen fuer Autos und " + getPersonen() + " Plaetzen fuer " +
	    "Personen\nBestehend aus:\n";

	//ist die Lok da?
	if(lok != null)
	{
	    //Daten der Lok ausgeben
	    ergebnis += lok;
	    ergebnis += "\nAnzahl der Waggons: " + waggons;
	    //sind Waggons da
	    waggon = lok.getAnfang();

	    //Daten aller Waggons anzeigen
	    if(waggon != null)
	    {
		ergebnis += "\n";
		ergebnis += waggon;
		
		while(waggon.getNaechster() != null)
		{
		    ergebnis += "\n";
		    ergebnis += waggon.getNaechster();

		    waggon = waggon.getNaechster();
		}
	    }
	}
	return ergebnis;
    }

    /** Main-Methode zum Testen
     */
    public static void main(String args[])
    {
	//unsere Lokomotive und der Zug zu dem sie gehrt
	Lokomotive chief = new Lokomotive("Big Chief", 23.0f, 5311);
	Zug santaFe = new Zug("Santa Fe", chief);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(santaFe);

	//Wagen anhngen und ausgeben, ob es geklappt hat
	santaFe.anhaengen(new Personenwaggon(25, 15f));
	santaFe.anhaengen(new Personenwaggon(125, 23f));
	
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(santaFe);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//Wagen abhngen und ausgeben, ob es geklappt hat
	santaFe.abhaengen();

	System.out.println(santaFe);
	
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//mehr Wagen abhngen, als da sind, sollte funktionieren und
	//keinen Fehler hervorrufen
	santaFe.abhaengen();
	santaFe.abhaengen();

	System.out.println(santaFe);
	
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	
	//ganz viel und unterschiedliches anhngen
	santaFe.anhaengen(new Personenwaggon(25, 15f));
	santaFe.anhaengen(new Personenwaggon(125, 23f));
	santaFe.anhaengen(new Autowaggon(20, 18.5f));
	santaFe.anhaengen(new Personenwaggon(125, 23f));
	santaFe.anhaengen(new Autowaggon(10, 14.5f));
	santaFe.anhaengen(new Autowaggon(15, 28.5f));
	santaFe.anhaengen(new Personenwaggon(15, 21f));

	System.out.println(santaFe);
	
	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//Zug umhngen
	//neuer Zug hat die Waggons des alten in der umgekehrten Reihenfolge
	//alter Zug keine Waggons mehr
	Lokomotive steam = new Lokomotive("Steam Waltz", 15.2f, 5332);
	Zug orient = new Zug("Orient Express", steam, santaFe);

	System.out.println(santaFe);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(orient);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	//Zug umhngen mit "leerem" Zug
	Lokomotive giant = new Lokomotive("Giant", 45.f, 5333);
	Zug orient2 = new Zug("Orient Express2", giant, santaFe);

	System.out.println(santaFe);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");

	System.out.println(orient2);

	System.out.println("\n----------------------------------------" + 
                           "----------------------------------------\n");
	
    }
}
